﻿#include "arcpainterinfo.h"
#include <QtMath>
#include <QDebug>

TITAN_BEGIN_NAMESPACE

class PcbCamArcPainterInfoData : public QSharedData
{
public:
    PcbCamArcPainterInfoData() { }
    PcbCamArcPainterInfoData(const PcbCamArcPainterInfoData &other): QSharedData(other){ }
    ~PcbCamArcPainterInfoData() { }

    QRectF rect;
    qreal startAngle {0.0};
    qreal spanAngle {0.0};
};

PcbCamArcPainterInfo::PcbCamArcPainterInfo(qreal xs, qreal ys, qreal xe, qreal ye, qreal xc, qreal yc, bool cw) : d(new PcbCamArcPainterInfoData)
{
    qreal sax = xs - xc, say = ys - yc;
    qreal eax = xe - xc, eay = ye - yc;

    qreal r = qSqrt(sax * sax + say * say);

    qreal sa = qAtan2(-say, sax);
    qreal ea = qAtan2(-eay, eax);

    if (cw) {
        if (ea <= sa) {
            ea += 2.0 * M_PI;
        }
    }
    else {
        if (sa <= ea) {
            sa += 2.0 * M_PI;
        }
    }

    d->rect = QRectF(xc - r, yc - r, r * 2.0, r * 2.0);
    d->startAngle = sa * (180.0 / M_PI) *16;
    d->spanAngle = (ea - sa) * (180.0 / M_PI) * 16;

}

PcbCamArcPainterInfo::PcbCamArcPainterInfo(const PcbCamArcPainterInfo &rhs) : d(rhs.d)
{

}

PcbCamArcPainterInfo &PcbCamArcPainterInfo::operator=(const PcbCamArcPainterInfo &rhs)
{
    if (this != &rhs)
        d.operator=(rhs.d);
    return *this;
}

PcbCamArcPainterInfo::~PcbCamArcPainterInfo()
{

}

QRectF PcbCamArcPainterInfo::rect() const
{
    return d->rect;
}

qreal PcbCamArcPainterInfo::startAngle() const
{
    return d->startAngle;
}

qreal PcbCamArcPainterInfo::spanAngle() const
{
    return d->spanAngle;
}

TITAN_END_NAMESPACE
